import { useFirestoreDoc, useFirestoreDocOnce, } from 'reactfire';
import { useDocumentSnapData } from './useSnapData.js';
import { useFirestoreErrorLogger, useSafeRef } from './utils.js';
const createUseTypedDocHook = (hook) => {
    return (typedDoc, dataOptions = {}) => {
        const { safeRef } = useSafeRef(typedDoc.raw);
        const { data: _snap, error, ...status } = hook(safeRef, { suspense: true });
        useFirestoreErrorLogger(error);
        const { snap, data } = useDocumentSnapData(typedDoc, _snap, dataOptions);
        return { snap, data, error, ...status };
    };
};
export const useTypedDoc = createUseTypedDocHook(useFirestoreDoc);
export const useTypedDocOnce = createUseTypedDocHook(useFirestoreDocOnce);
